@@ -17,6 +17,7 @@
 from functools import partial
 from operator import methodcaller
 import optparse
+import threading
 
 from extras import try_import
 v2_avail = try_import('subunit.ByteStreamToStreamResult')
@@ -29,6 +30,26 @@ from testrepository.commands import Command
 from testrepository.repository import RepositoryNotFound
 from testrepository.testcommand import TestCommand
 
+class InputToStreamResult(object):
+    """Generate Stream events from stdin.
+
+    Really a UI responsibility?
+    """
+
+    def __init__(self, stream):
+        self.source = stream
+        self.stop = False
+
+    def run(self, result):
+        while True:
+            if self.stop:
+                return
+            char = self.source.read(1)
+            if not char:
+                return
+            if char == b'a':
+                result.status(test_id='stdin', test_status='fail')
+
 
 class load(Command):
     """Load a subunit stream into a repository.
@@ -39,7 +60,7 @@ class load(Command):
     Unless the stream is a partial stream, any existing failures are discarded.
     """
 
-    input_streams = ['subunit+']
+    input_streams = ['subunit+', 'interactive?']
 
     args = [ExistingPathArgument('streams', min=0, max=None)]
     options = [
@@ -72,14 +93,14 @@ class load(Command):
         # back to it. Needs to be a callable - its a head fake for
         # testsuite.add.
         # XXX: Be nice if we could declare that the argument, which is a path,
-        # is to be an input stream.
+        # is to be an input stream - and thus push this conditional down into
+        # the UI object.
         if self.ui.arguments.get('streams'):
             opener = partial(open, mode='rb')
-            cases = lambda:map(opener, self.ui.arguments['streams'])
+            streams = map(opener, self.ui.arguments['streams'])
         else:
-            cases = lambda:self.ui.iter_streams('subunit')
-        def make_tests(suite):
-            streams = list(suite)
+            streams = self.ui.iter_streams('subunit')
+        def make_tests():
             for pos, stream in enumerate(streams):
                 if v2_avail:
                     # Calls StreamResult API.
@@ -104,7 +125,7 @@ class load(Command):
                     lambda result:testtools.StreamTagger(
                         [result], add=['worker-%d' % pos]))
                 yield (case, str(pos))
-        case = testtools.ConcurrentStreamTestSuite(lambda: list(make_tests(cases())))
+        case = testtools.ConcurrentStreamTestSuite(make_tests)
         # One unmodified copy of the stream to repository storage
         inserter = repo.get_inserter(partial=self.ui.options.partial)
         # One copy of the stream to the UI layer after performing global
@@ -116,11 +137,23 @@ class load(Command):
         output_result, summary_result = self.ui.make_result(
             inserter.get_id, testcommand, previous_run=previous_run)
         result = testtools.CopyStreamResult([inserter, output_result])
+        runner_thread = None
         result.startTestRun()
         try:
+            # Convert user input into a stdin event stream
+            interactive_streams = list(self.ui.iter_streams('interactive'))
+            if interactive_streams:
+                case = InputToStreamResult(interactive_streams[0])
+                runner_thread = threading.Thread(
+                    target=case.run, args=(result,))
+                runner_thread.daemon = True
+                runner_thread.start()
             case.run(result)
         finally:
             result.stopTestRun()
+            if interactive_streams and runner_thread:
+                runner_thread.stop = True
+                runner_thread.join(10)
         if not summary_result.wasSuccessful():
             return 1
         else:
