@@ -14,18 +14,21 @@
 
 """Persistent storage of test results."""
 
-from cStringIO import StringIO
+from io import BytesIO
 try:
     import anydbm as dbm
 except ImportError:
     import dbm
 import errno
+from operator import methodcaller
 import os.path
 import sys
 import tempfile
 
 import subunit
 from subunit import TestProtocolClient
+import testtools
+from testtools.compat import _b
 
 from testrepository.repository import (
     AbstractRepository,
@@ -48,7 +51,7 @@ class RepositoryFactory(AbstractRepositoryFactory):
         """Create a repository at url/path."""
         base = os.path.join(os.path.expanduser(url), '.testrepository')
         os.mkdir(base)
-        stream = file(os.path.join(base, 'format'), 'wb')
+        stream = open(os.path.join(base, 'format'), 'wt')
         try:
             stream.write('1\n')
         finally:
@@ -61,8 +64,8 @@ class RepositoryFactory(AbstractRepositoryFactory):
         path = os.path.expanduser(url)
         base = os.path.join(path, '.testrepository')
         try:
-            stream = file(os.path.join(base, 'format'), 'rb')
-        except (IOError, OSError), e:
+            stream = open(os.path.join(base, 'format'), 'rt')
+        except (IOError, OSError) as e:
             if e.errno == errno.ENOENT:
                 raise RepositoryNotFound(url)
             raise
@@ -96,7 +99,7 @@ class Repository(AbstractRepository):
         return value
 
     def _next_stream(self):
-        next_content = file(os.path.join(self.base, 'next-stream'), 'rb').read()
+        next_content = open(os.path.join(self.base, 'next-stream'), 'rt').read()
         try:
             return int(next_content)
         except ValueError:
@@ -113,19 +116,19 @@ class Repository(AbstractRepository):
  
     def get_failing(self):
         try:
-            run_subunit_content = file(
+            run_subunit_content = open(
                 os.path.join(self.base, "failing"), 'rb').read()
         except IOError:
             err = sys.exc_info()[1]
             if err.errno == errno.ENOENT:
-                run_subunit_content = ''
+                run_subunit_content = _b('')
             else:
                 raise
         return _DiskRun(None, run_subunit_content)
 
     def get_test_run(self, run_id):
         try:
-            run_subunit_content = file(
+            run_subunit_content = open(
                 os.path.join(self.base, str(run_id)), 'rb').read()
         except IOError as e:
             if e.errno == errno.ENOENT:
@@ -164,7 +167,7 @@ class Repository(AbstractRepository):
         # term. Likewise we don't fsync - this data isn't valuable enough to
         # force disk IO.
         prefix = self._path('next-stream')
-        stream = file(prefix + '.new', 'wb')
+        stream = open(prefix + '.new', 'wt')
         try:
             stream.write('%d\n' % value)
         finally:
@@ -179,18 +182,29 @@ class _DiskRun(AbstractTestRun):
         """Create a _DiskRun with the content subunit_content."""
         self._run_id = run_id
         self._content = subunit_content
+        assert type(subunit_content) is bytes
 
     def get_id(self):
         return self._run_id
 
     def get_subunit_stream(self):
-        return StringIO(self._content)
+        return BytesIO(self._content)
 
     def get_test(self):
-        return subunit.ProtocolTestCase(self.get_subunit_stream())
+        case = subunit.ProtocolTestCase(self.get_subunit_stream())
+        def wrap_result(result):
+            # Wrap in a router to mask out startTestRun/stopTestRun from the
+            # ExtendedToStreamDecorator.
+            result = testtools.StreamResultRouter(result, do_start_stop_run=False)
+            # Wrap that in ExtendedToStreamDecorator to convert v1 calls to
+            # StreamResult.
+            return testtools.ExtendedToStreamDecorator(result)
+        return testtools.DecorateTestCaseResult(
+            case, wrap_result, methodcaller('startTestRun'),
+            methodcaller('stopTestRun'))
 
 
-class _SafeInserter(TestProtocolClient):
+class _SafeInserter(object):
 
     def __init__(self, repository, partial=False):
         # XXX: Perhaps should factor into a decorator and use an unaltered
@@ -204,13 +218,25 @@ class _SafeInserter(TestProtocolClient):
         self._times = {}
         self._test_start = None
         self._time = None
-        TestProtocolClient.__init__(self, stream)
+        subunit_client = testtools.StreamToExtendedDecorator(
+            TestProtocolClient(stream))
+        self.hook = testtools.CopyStreamResult([
+            subunit_client,
+            testtools.StreamToDict(self._handle_test)])
+        self._stream = stream
+
+    def _handle_test(self, test_dict):
+        start, stop = test_dict['timestamps']
+        if None in (start, stop):
+            return
+        self._times[test_dict['id']] = str(timedelta_to_seconds(stop - start))
 
     def startTestRun(self):
-        pass
+        self.hook.startTestRun()
+        self._run_id = None
 
     def stopTestRun(self):
-        # TestProtocolClient.stopTestRun(self)
+        self.hook.stopTestRun()
         self._stream.flush()
         self._stream.close()
         run_id = self._name()
@@ -231,30 +257,18 @@ class _SafeInserter(TestProtocolClient):
                     db[key] = value
         finally:
             db.close()
-        return run_id
+        self._run_id = run_id
+
+    def status(self, *args, **kwargs):
+        self.hook.status(*args, **kwargs)
 
     def _cancel(self):
         """Cancel an insertion."""
         self._stream.close()
         os.unlink(self.fname)
 
-    def startTest(self, test):
-        result = TestProtocolClient.startTest(self, test)
-        self._test_start = self._time
-        return result
-
-    def stopTest(self, test):
-        result = TestProtocolClient.stopTest(self, test)
-        if None in (self._test_start, self._time):
-            return result
-        duration_seconds = timedelta_to_seconds(self._time - self._test_start)
-        self._times[test.id()] = str(duration_seconds)
-        return result
-
-    def time(self, timestamp):
-        result = TestProtocolClient.time(self, timestamp)
-        self._time = timestamp
-        return result
+    def get_id(self):
+        return self._run_id
 
 
 class _FailingInserter(_SafeInserter):
@@ -270,34 +284,36 @@ class _Inserter(_SafeInserter):
         return self._repository._allocate()
 
     def stopTestRun(self):
-        run_id = _SafeInserter.stopTestRun(self)
+        super(_Inserter, self).stopTestRun()
         # XXX: locking (other inserts may happen while we update the failing
         # file).
         # Combine failing + this run : strip passed tests, add failures.
         # use memory repo to aggregate. a bit awkward on layering ;).
-        import memory
+        # Should just pull the failing items aside as they happen perhaps.
+        # Or use a router and avoid using a memory object at all.
+        from testrepository.repository import memory
         repo = memory.Repository()
         if self.partial:
             # Seed with current failing
-            inserter = repo.get_inserter()
+            inserter = testtools.ExtendedToStreamDecorator(repo.get_inserter())
             inserter.startTestRun()
             failing = self._repository.get_failing()
             failing.get_test().run(inserter)
             inserter.stopTestRun()
-        inserter= repo.get_inserter(partial=True)
+        inserter= testtools.ExtendedToStreamDecorator(repo.get_inserter(partial=True))
         inserter.startTestRun()
-        run = self._repository.get_test_run(run_id)
+        run = self._repository.get_test_run(self.get_id())
         run.get_test().run(inserter)
         inserter.stopTestRun()
         # and now write to failing
         inserter = _FailingInserter(self._repository)
-        inserter.startTestRun()
+        _inserter = testtools.ExtendedToStreamDecorator(inserter)
+        _inserter.startTestRun()
         try:
-            try:
-                repo.get_failing().get_test().run(inserter)
-            except:
-                inserter._cancel()
-                raise
-        finally:
-            inserter.stopTestRun()
-        return run_id
+            repo.get_failing().get_test().run(_inserter)
+        except:
+            inserter._cancel()
+            raise
+        else:
+            _inserter.stopTestRun()
+        return self.get_id()
