@@ -14,7 +14,9 @@
 
 """In memory storage of test results."""
 
-from collections import OrderedDict
+from extras import try_import
+
+OrderedDict = try_import('collections.OrderedDict', dict)
 from io import BytesIO
 from operator import methodcaller
 
