@@ -16,6 +16,7 @@
 
 from extras import try_imports
 
+from collections import defaultdict
 ConfigParser = try_imports(['ConfigParser', 'configparser'])
 import itertools
 import operator
@@ -24,12 +25,11 @@ import re
 import subprocess
 import sys
 import tempfile
+import multiprocessing
 from textwrap import dedent
 
 from fixtures import Fixture
-from testtools.compat import _b
 
-from testrepository.results import TestResultFilter
 from testrepository.testlist import (
     parse_enumeration,
     write_list,
@@ -73,6 +73,7 @@ testrconf_help = dedent("""
       be adjusted if the paths are synched with different names.
     * instance_dispose -- dispose of one or more test running environments.
       Accepts $INSTANCE_IDS.
+    * group_regex -- If set group tests by the matched section of the test id.
     * $IDOPTION -- the variable to use to trigger running some specific tests.
     * $IDFILE -- A file created before the test command is run and deleted
       afterwards which contains a list of test ids, one per line. This can
@@ -135,7 +136,7 @@ class TestListingFixture(Fixture):
 
     def __init__(self, test_ids, cmd_template, listopt, idoption, ui,
         repository, parallel=True, listpath=None, parser=None,
-        test_filters=None, instance_source=None):
+        test_filters=None, instance_source=None, group_callback=None):
         """Create a TestListingFixture.
 
         :param test_ids: The test_ids to use. May be None indicating that
@@ -168,6 +169,10 @@ class TestListingFixture(Fixture):
         :param instance_source: A source of test run instances. Must support
             obtain_instance(max_concurrency) -> id and release_instance(id)
             calls.
+        :param group_callback: If supplied, should be a function that accepts a
+            test id and returns a group id. A group id is an arbitrary value
+            used as a dictionary key in the scheduler. All test ids with the
+            same group id are scheduled onto the same backend test process.
         """
         self.test_ids = test_ids
         self.template = cmd_template
@@ -179,6 +184,7 @@ class TestListingFixture(Fixture):
         self._listpath = listpath
         self._parser = parser
         self.test_filters = test_filters
+        self._group_callback = group_callback
         self._instance_source = instance_source
 
     def setUp(self):
@@ -287,7 +293,10 @@ class TestListingFixture(Fixture):
             run_proc = self.ui.subprocess_Popen(list_cmd, shell=True,
                 stdout=subprocess.PIPE, stdin=subprocess.PIPE)
             out, err = run_proc.communicate()
-            # Should we raise on non-zero exit?
+            if run_proc.returncode != 0:
+                raise ValueError(
+                    "Non-zero exit code (%d) from test listing."
+                    " stdout=%r, stderr=%r" % (run_proc.returncode, out, err))
             ids = parse_enumeration(out)
             return ids
         finally:
@@ -371,22 +380,55 @@ class TestListingFixture(Fixture):
         partitions = [list() for i in range(concurrency)]
         timed_partitions = [[0.0, partition] for partition in partitions]
         time_data = self.repository.get_test_times(test_ids)
-        timed = time_data['known']
-        unknown = time_data['unknown']
+        timed_tests = time_data['known']
+        unknown_tests = time_data['unknown']
+        # Group tests: generate group_id -> test_ids.
+        group_ids = defaultdict(list)
+        if self._group_callback is None:
+            group_callback = lambda _:None
+        else:
+            group_callback = self._group_callback
+        for test_id in test_ids:
+            group_id = group_callback(test_id) or test_id
+            group_ids[group_id].append(test_id)
+        # Time groups: generate three sets of groups:
+        # - fully timed dict(group_id -> time),
+        # - partially timed dict(group_id -> time) and
+        # - unknown (set of group_id)
+        # We may in future treat partially timed different for scheduling, but
+        # at least today we just schedule them after the fully timed groups.
+        timed = {}
+        partial = {}
+        unknown = []
+        for group_id, group_tests in group_ids.items():
+            untimed_ids = unknown_tests.intersection(group_tests)
+            group_time = sum([timed_tests[test_id]
+                for test_id in untimed_ids.symmetric_difference(group_tests)])
+            if not untimed_ids:
+                timed[group_id] = group_time
+            elif group_time:
+                partial[group_id] = group_time
+            else:
+                unknown.append(group_id)
         # Scheduling is NP complete in general, so we avoid aiming for
         # perfection. A quick approximation that is sufficient for our general
         # needs:
-        # sort the tests by time
-        # allocate to partitions by putting each test in to the partition with
-        # the current (lowest time, shortest length)
-        queue = sorted(timed.items(), key=operator.itemgetter(1), reverse=True)
-        for test_id, duration in queue:
-            timed_partitions[0][0] = timed_partitions[0][0] + duration
-            timed_partitions[0][1].append(test_id)
-            timed_partitions.sort(key=lambda item:(item[0], len(item[1])))
-        # Assign tests with unknown times in round robin fashion to the partitions.
-        for partition, test_id in zip(itertools.cycle(partitions), unknown):
-            partition.append(test_id)
+        # sort the groups by time
+        # allocate to partitions by putting each group in to the partition with
+        # the current (lowest time, shortest length[in tests])
+        def consume_queue(groups):
+            queue = sorted(
+                groups.items(), key=operator.itemgetter(1), reverse=True)
+            for group_id, duration in queue:
+                timed_partitions[0][0] = timed_partitions[0][0] + duration
+                timed_partitions[0][1].extend(group_ids[group_id])
+                timed_partitions.sort(key=lambda item:(item[0], len(item[1])))
+        consume_queue(timed)
+        consume_queue(partial)
+        # Assign groups with entirely unknown times in round robin fashion to
+        # the partitions. 
+        for partition, group_id in zip(itertools.cycle(partitions), unknown):
+            partition.extend(group_ids[group_id])
         return partitions
 
     def callout_concurrency(self):
@@ -406,14 +448,11 @@ class TestListingFixture(Fixture):
         return int(out.strip())
 
     def local_concurrency(self):
-        if sys.platform == 'linux2':
-            concurrency = None
-            for line in open('/proc/cpuinfo', 'rt'):
-                if line.startswith('processor'):
-                    concurrency = int(line[line.find(':')+1:]) + 1
-            return concurrency
-        # No concurrency logic known.
-        return None
+        try:
+            return multiprocessing.cpu_count()
+        except NotImplementedError:
+            # No concurrency logic known.
+            return None
 
 
 class TestCommand(Fixture):
@@ -516,15 +555,28 @@ class TestCommand(Fixture):
                 if e.message != "No option 'test_list_option' in section: 'DEFAULT'":
                     raise
                 raise ValueError("No test_list_option option present in .testr.conf")
+        try:
+            group_regex = parser.get('DEFAULT', 'group_regex')
+        except ConfigParser.NoOptionError:
+            group_regex = None
+        if group_regex:
+            def group_callback(test_id, regex=re.compile(group_regex)):
+                match = regex.match(test_id)
+                if match:
+                    return match.group(0)
+        else:
+            group_callback = None
         if self.oldschool:
             listpath = os.path.join(self.ui.here, 'failing.list')
             result = self.run_factory(test_ids, cmd, listopt, idoption,
                 self.ui, self.repository, listpath=listpath, parser=parser,
-                test_filters=test_filters, instance_source=self)
+                test_filters=test_filters, instance_source=self,
+                group_callback=group_callback)
         else:
             result = self.run_factory(test_ids, cmd, listopt, idoption,
                 self.ui, self.repository, parser=parser,
-                test_filters=test_filters, instance_source=self)
+                test_filters=test_filters, instance_source=self,
+                group_callback=group_callback)
         return result
 
     def get_filter_tags(self):
@@ -568,39 +620,6 @@ class TestCommand(Fixture):
         self._allocated_instances.add(result)
         return result
 
-    def make_result(self, receiver):
-        """Create a TestResult that will perform any global filtering etc.
-
-        :param receiver: The result to forward the result of global filtering.
-        :return: A TestResult.
-        """
-        filter_tags = self.get_filter_tags()
-        if filter_tags:
-            try:
-                from subunit.test_results import make_tag_filter
-            except ImportError:
-                raise ValueError(
-                    "Subunit not installed or does not have tag filtering support")
-            # predicates return False to filter something out. We want to
-            # filter out tagged tests *unless* they fail/error. So we want
-            # tag_p:False + outcome_p:False -> False
-            # tag_p:False + outcome_p:True -> True
-            # tag_p:True + * -> True
-            def error_or_fail(t, outcome, e, d, tags):
-                return outcome in ('error', 'failure')
-            def or_predicates(predicates):
-                def fn(*args, **kwargs):
-                    for predicate in predicates:
-                        if predicate(*args, **kwargs):
-                            return True
-                    return False
-                return fn
-            predicates = [make_tag_filter(None, filter_tags), error_or_fail]
-            predicate = or_predicates(predicates)
-            return TestResultFilter(
-                receiver, filter_success=False, filter_predicate=predicate)
-        return receiver
-
     def release_instance(self, instance_id):
         """Return instance_ids to the pool for reuse."""
         self._allocated_instances.remove(instance_id)
