@@ -54,13 +54,13 @@ class TestCommand(ResourcedTestCase):
              [('id', id, None), ('failures', 1, None)])],
             ui.outputs)
         suite = ui.outputs[0][1]
-        result = testtools.TestResult()
+        result = testtools.StreamSummary()
         result.startTestRun()
         try:
             suite.run(result)
         finally:
             result.stopTestRun()
-        self.assertEqual(1, len(result.failures))
+        self.assertEqual(1, len(result.errors))
         self.assertEqual(2, result.testsRun)
 
     def _add_run(self, repo):
@@ -85,28 +85,15 @@ class TestCommand(ResourcedTestCase):
              [('id', id, None), ('failures', 1, 0)])],
             ui.outputs)
         suite = ui.outputs[0][1]
-        result = testtools.TestResult()
+        result = testtools.StreamSummary()
         result.startTestRun()
         try:
             suite.run(result)
         finally:
             result.stopTestRun()
-        self.assertEqual(1, len(result.failures))
+        self.assertEqual(1, len(result.errors))
         self.assertEqual(2, result.testsRun)
 
-    def test_grabs_TestCommand_result(self):
-        ui, cmd = self.get_test_ui_and_cmd()
-        cmd.repository_factory = memory.RepositoryFactory()
-        repo = cmd.repository_factory.initialise(ui.here)
-        inserter = repo.get_inserter()
-        inserter.startTestRun()
-        inserter.stopTestRun()
-        cmd.command_factory = StubTestCommand()
-        cmd.execute()
-        self.assertEqual(
-            [('startTestRun',), ('stopTestRun',)],
-            cmd.command_factory.results[0]._events)
-
     def test_shows_subunit_stream(self):
         ui, cmd = self.get_test_ui_and_cmd(options=[('subunit', True)])
         cmd.repository_factory = memory.RepositoryFactory()
