@@ -221,6 +221,16 @@ class TestTestCommand(ResourcedTestCase):
         expected_cmd = 'foo '
         self.assertEqual(expected_cmd, fixture.cmd)
 
+    def test_group_regex_option(self):
+        ui, command = self.get_test_ui_and_cmd()
+        self.set_config(
+                '[DEFAULT]\ntest_command=foo $IDOPTION\n'
+                'test_id_option=--load-list $IDFILE\n'
+                'group_regex=([^\\.]+\\.)+\n')
+        fixture = self.useFixture(command.get_run_command())
+        self.assertEqual(
+            'pkg.class.', fixture._group_callback('pkg.class.test_method'))
+
     def test_extra_args_passed_in(self):
         ui, command = self.get_test_ui_and_cmd()
         self.set_config(
@@ -311,6 +321,15 @@ class TestTestCommand(ResourcedTestCase):
         fixture = self.useFixture(command.get_run_command())
         self.assertEqual(set(['returned', 'ids']), set(fixture.list_tests()))
 
+    def test_list_tests_nonzero_exit(self):
+        ui, command = self.get_test_ui_and_cmd()
+        ui.proc_results = [1]
+        self.set_config(
+            '[DEFAULT]\ntest_command=foo $LISTOPT $IDLIST\ntest_id_list_default=whoo yea\n'
+            'test_list_option=--list\n')
+        fixture = self.useFixture(command.get_run_command())
+        self.assertThat(lambda:fixture.list_tests(), raises(ValueError))
+
     def test_partition_tests_smoke(self):
         repo = memory.RepositoryFactory().initialise('memory:')
         # Seed with 1 slow and 2 tests making up 2/3 the time.
@@ -362,6 +381,46 @@ class TestTestCommand(ResourcedTestCase):
         self.assertEqual(1, len(partitions[0]))
         self.assertEqual(1, len(partitions[1]))
 
+    def test_partition_tests_with_grouping(self):
+        repo = memory.RepositoryFactory().initialise('memory:')
+        result = repo.get_inserter()
+        result.startTestRun()
+        run_timed("TestCase1.slow", 3, result)
+        run_timed("TestCase2.fast1", 1, result)
+        run_timed("TestCase2.fast2", 1, result)
+        result.stopTestRun()
+        ui, command = self.get_test_ui_and_cmd(repository=repo)
+        self.set_config(
+            '[DEFAULT]\ntest_command=foo $IDLIST $LISTOPT\n'
+            'test_list_option=--list\n')
+        fixture = self.useFixture(command.get_run_command())
+        test_ids = frozenset(['TestCase1.slow', 'TestCase1.fast',
+                              'TestCase1.fast2', 'TestCase2.fast1',
+                              'TestCase3.test1', 'TestCase3.test2',
+                              'TestCase2.fast2', 'TestCase4.test',
+                              'testdir.testfile.TestCase5.test'])
+        regex = 'TestCase[0-5]'
+        def group_id(test_id, regex=re.compile('TestCase[0-5]')):
+            match = regex.match(test_id)
+            if match:
+                return match.group(0)
+        # There isn't a public way to define a group callback [as yet].
+        fixture._group_callback = group_id
+        partitions = fixture.partition_tests(test_ids, 2)
+        # Timed groups are deterministic:
+        self.assertTrue('TestCase2.fast1' in partitions[0])
+        self.assertTrue('TestCase2.fast2' in partitions[0])
+        self.assertTrue('TestCase1.slow' in partitions[1])
+        self.assertTrue('TestCase1.fast' in partitions[1])
+        self.assertTrue('TestCase1.fast2' in partitions[1])
+        # Untimed groups just need to be kept together:
+        if 'TestCase3.test1' in partitions[0]:
+            self.assertTrue('TestCase3.test2' in partitions[0])
+        if 'TestCase4.test' not in partitions[0]:
+            self.assertTrue('TestCase4.test' in partitions[1])
+        if 'testdir.testfile.TestCase5.test' not in partitions[0]:
+            self.assertTrue('testdir.testfile.TestCase5.test' in partitions[1])
+
     def test_run_tests_with_instances(self):
         # when there are instances and no instance_execute, run_tests acts as
         # normal.
@@ -488,52 +547,6 @@ class TestTestCommand(ResourcedTestCase):
         self.assertEqual(None, fixture.callout_concurrency())
         self.assertEqual([], ui.outputs)
 
-    def test_make_result(self):
-        # Just a simple 'the dots are joined' test. More later.
-        ui, command = self.get_test_ui_and_cmd()
-        log = ExtendedTestResult()
-        self.set_config('[DEFAULT]\n')
-        result = command.make_result(log)
-        result.startTestRun()
-        result.stopTestRun()
-        self.assertEqual([('startTestRun',), ('stopTestRun',)], log._events)
-
-    def test_make_result_tag_filter(self):
-        ui, command = self.get_test_ui_and_cmd()
-        self.set_config('[DEFAULT]\nfilter_tags=foo bar\n')
-        log = ExtendedTestResult()
-        class Tests(ResourcedTestCase):
-            def ignored(self): pass
-            def fails(self): self.fail('foo')
-            def filtered(self): pass
-        try:
-            result = command.make_result(log)
-        except ValueError:
-            self.skip("Subunit too old for tag filtering support.")
-        result.startTestRun()
-        result.tags(set(['ignored']), set())
-        ignored = Tests("ignored")
-        ignored.run(result)
-        result.tags(set(['foo']), set())
-        fails = Tests("fails")
-        fails.run(result)
-        filtered = Tests("filtered")
-        filtered.run(result)
-        result.stopTestRun()
-        self.assertEqual([
-            ('startTestRun',),
-            ('tags', set(['ignored']), set()),
-            ('startTest', ignored),
-            ('addSuccess', ignored),
-            ('stopTest', ignored),
-            ('tags', set(['foo']), set()),
-            ('startTest', fails),
-            ('addFailure', fails, Wildcard),
-            ('stopTest', fails),
-            ('stopTestRun',),
-            ],
-            log._events)
-
     def test_filter_tests_by_regex_only(self):
         if v2_avail:
             buffer = BytesIO()
