@@ -14,11 +14,13 @@
 
 """Tests for UI support logic and the UI contract."""
 
-from cStringIO import StringIO
+from io import BytesIO, TextIOWrapper
 import optparse
 import subprocess
 import sys
 
+from fixtures import EnvironmentVariable
+from testtools.compat import _b, _u
 from testtools.content import text_content
 from testtools.matchers import raises
 
@@ -35,12 +37,12 @@ def cli_ui_factory(input_streams=None, options=(), args=()):
         # something - however, may need to be cli specific tests at that
         # point.
         raise NotImplementedError(cli_ui_factory)
-    stdout = StringIO()
+    stdout = TextIOWrapper(BytesIO(), line_buffering=True)
     if input_streams:
-        stdin = StringIO(input_streams[0][1])
+        stdin = TextIOWrapper(BytesIO(input_streams[0][1]))
     else:
-        stdin = StringIO()
-    stderr = StringIO()
+        stdin = TextIOWrapper(BytesIO())
+    stderr = TextIOWrapper(BytesIO(), line_buffering=True)
     argv = list(args)
     for option, value in options:
         # only bool handled so far
@@ -76,7 +78,7 @@ class TestUIContract(ResourcedTestCase):
         ui = self.ui_factory()
 
     def test_factory_input_stream_args(self):
-        ui = self.ui_factory([('subunit', 'value')])
+        ui = self.ui_factory([('subunit', _b('value'))])
 
     def test_here(self):
         ui = self.get_test_ui()
@@ -85,20 +87,21 @@ class TestUIContract(ResourcedTestCase):
     def test_iter_streams_load_stdin_use_case(self):
         # A UI can be asked for the streams that a command has indicated it
         # accepts, which is what load < foo will require.
-        ui = self.ui_factory([('subunit', 'test: foo\nsuccess: foo\n')])
+        ui = self.ui_factory([('subunit', _b('test: foo\nsuccess: foo\n'))])
         cmd = commands.Command(ui)
         cmd.input_streams = ['subunit+']
         ui.set_command(cmd)
         results = []
         for result in ui.iter_streams('subunit'):
             results.append(result.read())
-        self.assertEqual(['test: foo\nsuccess: foo\n'], results)
+        self.assertEqual([_b('test: foo\nsuccess: foo\n')], results)
 
     def test_iter_streams_unexpected_type_raises(self):
         ui = self.get_test_ui()
         self.assertThat(lambda: ui.iter_streams('subunit'), raises(KeyError))
 
     def test_output_error(self):
+        self.useFixture(EnvironmentVariable('TESTR_PDB'))
         try:
             raise Exception('fooo')
         except Exception:
@@ -109,12 +112,12 @@ class TestUIContract(ResourcedTestCase):
     def test_output_rest(self):
         # output some ReST - used for help and docs.
         ui = self.get_test_ui()
-        ui.output_rest('')
+        ui.output_rest(_u(''))
 
     def test_output_stream(self):
         # a stream of bytes can be output.
         ui = self.get_test_ui()
-        ui.output_stream(StringIO())
+        ui.output_stream(BytesIO())
 
     def test_output_table(self):
         # output_table shows a table.
@@ -223,21 +226,23 @@ class TestUIContract(ResourcedTestCase):
         out, err = proc.communicate()
 
     def test_make_result(self):
-        # make_result should return a TestResult.
+        # make_result should return a StreamResult and a summary result.
         ui = self.ui_factory()
         ui.set_command(commands.Command(ui))
-        result = ui.make_result(lambda: None, StubTestCommand())
+        result, summary = ui.make_result(lambda: None, StubTestCommand())
         result.startTestRun()
+        result.status()
         result.stopTestRun()
-        self.assertEqual(0, result.testsRun)
+        summary.wasSuccessful()
 
     def test_make_result_previous_run(self):
         # make_result can take a previous run.
         ui = self.ui_factory()
         ui.set_command(commands.Command(ui))
-        result = ui.make_result(
+        result, summary = ui.make_result(
             lambda: None, StubTestCommand(),
             previous_run=memory.Repository().get_failing())
         result.startTestRun()
+        result.status()
         result.stopTestRun()
-        self.assertEqual(0, result.testsRun)
+        summary.wasSuccessful()
