@@ -28,21 +28,29 @@ from testtools.compat import unicode_output_stream, _u
 
 from testrepository import ui
 from testrepository.commands import get_command_parser
-from testrepository.results import TestResultFilter
 
 
 class CLITestResult(ui.BaseUITestResult):
     """A TestResult for the CLI."""
 
-    def __init__(self, ui, get_id, stream, previous_run=None):
-        """Construct a CLITestResult writing to stream."""
+    def __init__(self, ui, get_id, stream, previous_run=None, filter_tags=None):
+        """Construct a CLITestResult writing to stream.
+        
+        :param filter_tags: Tags that should be used to filter tests out. When
+            a tag in this set is present on a test outcome, the test is not
+            counted towards the test run count. If the test errors, then it is
+            still counted and the error is still shown.
+        """
         super(CLITestResult, self).__init__(ui, get_id, previous_run)
         self.stream = unicode_output_stream(stream)
         self.sep1 = _u('=' * 70 + '\n')
         self.sep2 = _u('-' * 70 + '\n')
+        self.filter_tags = filter_tags or frozenset()
+        self.filterable_states = set(['success', 'uxsuccess', 'xfail', 'skip'])
 
-    def _format_error(self, label, test, error_text):
-        tags = _u(' ').join(self.current_tags)
+    def _format_error(self, label, test, error_text, test_tags=None):
+        test_tags = test_tags or ()
+        tags = _u(' ').join(test_tags)
         if tags:
             tags = _u('tags: %s\n') % tags
         return _u('').join([
@@ -53,13 +61,21 @@ class CLITestResult(ui.BaseUITestResult):
             error_text,
             ])
 
-    def addError(self, test, err=None, details=None):
-        super(CLITestResult, self).addError(test, err=err, details=details)
-        self.stream.write(self._format_error(_u('ERROR'), *(self.errors[-1])))
-
-    def addFailure(self, test, err=None, details=None):
-        super(CLITestResult, self).addFailure(test, err=err, details=details)
-        self.stream.write(self._format_error(_u('FAIL'), *(self.failures[-1])))
+    def status(self, test_id=None, test_status=None, test_tags=None,
+        runnable=True, file_name=None, file_bytes=None, eof=False,
+        mime_type=None, route_code=None, timestamp=None):
+        super(CLITestResult, self).status(test_id=test_id,
+            test_status=test_status, test_tags=test_tags, runnable=runnable,
+            file_name=file_name, file_bytes=file_bytes, eof=eof,
+            mime_type=mime_type, route_code=route_code, timestamp=timestamp)
+        if test_status == 'fail':
+            self.stream.write(
+                self._format_error(_u('FAIL'), *(self._summary.errors[-1]),
+                test_tags=test_tags))
+        if test_status not in self.filterable_states:
+            return
+        if test_tags and test_tags.intersection(self.filter_tags):
+            self._summary.testsRun -= 1
 
 
 class UI(ui.AbstractUI):
@@ -79,6 +95,13 @@ class UI(ui.AbstractUI):
         self._stderr = stderr
 
     def _iter_streams(self, stream_type):
+        # Only the first stream declared in a command can be accepted at the
+        # moment - as there is only one stdin and alternate streams are not yet
+        # configurable in the CLI.
+        first_stream_type = self.cmd.input_streams[0]
+        if (stream_type != first_stream_type
+            and stream_type != first_stream_type[:-1]):
+            return
         yield subunit.make_stream_binary(self._stdin)
 
     def make_result(self, get_id, test_command, previous_run=None):
@@ -88,18 +111,20 @@ class UI(ui.AbstractUI):
             else:
                 serializer = StreamToExtendedDecorator(
                     subunit.TestProtocolClient(self._stdout))
-            # By pass user transforms - just forward it all and interpret
-            # everything as success.
-            result = ExtendedToStreamDecorator(serializer)
+            # By pass user transforms - just forward it all,
+            result = serializer
+            # and interpret everything as success.
             summary = testtools.StreamSummary()
             summary.startTestRun()
             summary.stopTestRun()
             return result, summary
         else:
-            output = CLITestResult(self, get_id, self._stdout, previous_run)
             # Apply user defined transforms.
-            result = ExtendedToStreamDecorator(StreamToExtendedDecorator(test_command.make_result(output)))
-        return result, result
+            filter_tags = test_command.get_filter_tags()
+            output = CLITestResult(self, get_id, self._stdout, previous_run,
+                filter_tags=filter_tags)
+            summary = output._summary
+        return output, summary
 
     def output_error(self, error_tuple):
         if 'TESTR_PDB' in os.environ:
